"""
"""

goto_host_funct_ = "goto_host"
goto_service_funct_ = "goto_service"
goto_repo_func_ = "goto_repo"

list_host_func_ = "list_host"
list_service_func_ = "list_service"

desc_commit_func_ = "desc_commit"
